#!/usr/bin/perl

######################################################################
#
# Configuration UI
# Xymon/Hobbit/BigBrother client for Synology DiskStation
# http://6co.ch/synomon/
#
# Donation are always welcome! Please check http://6co.ch/synomon/
# and you will find the magic button ;-)
#
# The Synology Monitoring tool is a Perl client for Xymon/Hobbit/BigBrother
# server that is tuned to work with Synology devices
#
#
# LICENCE
#
#   Copyright (c) 2012-2019, SysCo systemes de communication sa
#   SysCo (tm) is a trademark of SysCo systemes de communication sa
#   (http://www.sysco.ch/)
#   All rights reserved.
# 
#   This file is part of the Synology Monitoring Tool
#
#   Synology Monitoring Tool is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public License as
#   published by the Free Software Foundation, either version 3 of the License,
#   or (at your option) any later version.
# 
#   Synology Monitoring Tool is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Lesser General Public License for more details.
# 
#   You should have received a copy of the GNU Lesser General Public
#   License along with Synology Monitoring Tool.
#   If not, see <http://www.gnu.org/licenses/>.
#
#
# @author: SysCo/al
# @since CreationDate: 2012-11-14
# @copyright (c) 2012-2019 by SysCo systemes de communication sa
# @version $LastChangedRevision: 1.5.3 $
# @version $LastChangedDate: 2019-11-23 $
# @version $LastChangedBy: SysCo/al $
# @link $HeadURL: start-stop-status $
# @link http://6co.ch/synomon/
# @link developer@sysco.ch
# Language: Perl (already embedded in Synology devices)
#
#
# Change Log
#
#   2019-11-23 1.5.3  SysCo/al Fix to avoid crashing xymond (after upgrade to XyMon 4.3.30, thanks to Henk.
#                              Internal IP address(es) added in the general info header
#   2019-11-08 1.5.1  SysCo/al Detailed information reorganized to put Linux version at the first line
#   2017-01-05 1.4.8  SysCo/al Updated distribution
#   2017-01-04 1.4.7  SysCo/al Added CGI.pm and CGI/Util.pm (removed in the current Perl distribution)
#   2017-01-04 1.4.6  SysCo/al Remove Perl switch usage (also for index.cgi)
#   2017-01-04 1.4.5  SysCo/al Remove Perl switch usage (not supported in the last Perl package from Synology)
#   2016-04-07 1.4.2  SysCo/al Better disk handling (no red warning with usb drives), better UPS status handling
#   2016-04-02 1.4.1  SysCo/al Better DSM 6 support (enhanced raid, procs and general info)
#   2016-04-02 1.4.0  SysCo/al Better DSM 6 support (enhanced procs info)
#   2016-03-31 1.3.3  SysCo/al DSM 6 support (CPU was not monitored anymore)
#   2014-02-24 1.3.0  SysCo/lz Now all volumes are monitored
#   2013-03-05 1.2.4  SysCo/al Minor UI update
#   2012-12-15 1.2.0  SysCo/al Adding UPS support
#   2012-12-01 1.1.2  SysCo/al More options in the UI
#   2012-11-28 1.1.1  SysCo/al Updated UI
#   2012-11-28 1.1.0  SysCo/al Adding local history file option for history and debbuging purposes
#                              Better start/stop of the cron daemon in the start-stop-status file
#                              More generic script (for other packages)
#   2012-11-14 0.9.2  SysCo/al Installation issues
#
#####################################################################/

# Various path definitions
#
my $PackageName="SynoMon";
my $PackageNiceName="Synology Monitoring Tool";
my $PackageFileName="synomon";
my $PackageApp="/var/packages/$PackageName/target/app";
my $PackageAppConf="/etc/$PackageFileName.ini";
my $PackageAppLog=$PackageApp."/debug.log";
my $VarLogFolder="/var/log";
my $PackageHistoryFile="$VarLogFolder/$PackageFileName.csv";

# Default values
#
my $my_hostname = read_file('/proc/sys/kernel/hostname');
my $server_ip = "";
my $server_port = 1984;

my $local_history = 0;
my $history_separator = ";";

my $cpu_yellow = 6.0;
my $cpu_red = 12.0;

my $disk_yellow = 85;
my $disk_red = 95;

my $percent_physical_yellow = 99;
my $percent_physical_red = 100;
my $percent_actual_yellow = 70;
my $percent_actual_red = 90;
my $percent_swap_yellow = 80;
my $percent_swap_red = 90;

my $uptime_yellow = 30*60;


my $new_params = 0;

use CGI qw(param);
use Socket;

sub read_file
{
    my $file = shift;
    my $ret_file;
    open(FILE, "<$file") || return "unknown";
    while (<FILE>)
    {
        chomp;
        $ret_file .= $_;
    }
    close(FILE);
    return $ret_file;
}

print "Content-type: text/html\; charset=utf-8\n\n";

$lang_example = "example: ";
$lang_package_info = "$PackageNiceName is a package to monitor the Synology ressources, volumes and disks, sending the monitored information every 5 minutes to a XyMon/Hobbit/BigBrother compatible monitoring server.";
$lang_package_center_info = "To check automatically for updates for this package, add <b>Name: SysCo Synology Packages</b> and <b>Location: http://synology.sysco.ch</b> as a location in <b>Package Center - Settings - Package Sources - Add</b>";
$lang_login_error_text = "Please login as admin first, before using this webpage";
$lang_title = "$PackageNiceName";
$lang_update_configuration = "Update";
$lang_url = "http://6co.ch/synomon/";

$lang_local_history = "Keep a local history file";
$lang_history_separator = "Separator in the history file";
$lang_history_link = "Open the history file";
$lang_history_clear = "Clear the history file";

$lang_enter_hostname = "Enter the hostname to be displayed for this device";
$lang_enter_server_port = "Enter the monitoring server destination port";

$lang_hostname = "Hostname for this device";
$lang_server_ip = "Monitoring server IP or FQDN";
$lang_server_port = "Monitoring server port";
$lang_cpu_yellow = "CPU load for a yellow alert";
$lang_cpu_red = "CPU load for a red alert";
$lang_disk_yellow = "Disk used capacity for a yellow alert";
$lang_disk_red = "Disk used capacity for a red alert";
$lang_percent_physical_yellow = "Physical memory used for a yellow alert";
$lang_percent_physical_red = "Physical memory used for a red alert";
$lang_percent_actual_yellow = "Actual memory used for a yellow alert";
$lang_percent_actual_red = "Actual memory used for a red alert";
$lang_percent_swap_yellow = "Swap memory used for a yellow alert";
$lang_percent_swap_red = "Swap memory used for a red alert";
$lang_uptime_yellow = "Minimum time (in seconds) before uptime turn back to green";
    
if (index($ENV{HTTP_ACCEPT_LANGUAGE},"fr") == 0)
{
    $lang_local_history = "Conserver un fichier d'historique local";
    $lang_history_separator = "Séparateur dans le fichier d'historique";
    $lang_history_link = "Ouvrir le fichier d'historique";
    $lang_history_clear = "Effacer le fichier d'historique";

    $lang_enter_hostname = "Saisir le nom d'hôte qui sera affiché pour cet appareil";
    $lang_enter_server_port = "Entrer le port de destination du serveur de monitoring";

    $lang_hostname = "Nom d'hôte pour cet appareil";
    $lang_server_ip = "Adresse IP ou FQDN du serveur de monitoring";
    $lang_server_port = "Port du serveur de monitoring";
    
	$lang_example = "exemple: ";
    $lang_package_info = "$PackageNiceName est un paquet pour surveiller les ressources, les volumes et les disques d'une baie Synology en envoyant les informations surveillées toutes les 5 minutes à un serveur de surveillance compatible XyMon/Hobbit/BigBrother.";
$lang_package_center_info = "Pour vérifier automatiquement les mises à jour de ce paquet, ajouter <b>Nom: SysCo Synology Packages</b> et <b>Emplacement: http://synology.sysco.ch</b> comme emplacement dans <b>Centre de paquets - Paramètres - Sources de paquet - Ajouter</b>";
	$lang_login_error = "Authentification requise";
	$lang_login_error_text = "Il est nécessaire de se connecter en tant qu'admin avant d'utiliser cette page";
	$lang_title = "$PackageNiceName";
    $lang_update_configuration = "Mettre à jour";
}

# Are we authenticated yet ?
if (open (IN,"/usr/syno/synoman/webman/modules/authenticate.cgi|"))
{
	$user=<IN>;
	chop($user);
	close(IN);
}

# if no reply..no user...no authentication...so, bye-bye
#if (!$user)
#{
#	print "<html><head><title>".$lang_login_error."</title></head><body>".$lang_login_error_text."</body></html>\n";
#	die;
#}

# if configuration file exist, loading the configuration
if (-e $PackageAppConf)
{
    my $keyword;
    my $value;
    open(FILE, "<$PackageAppConf");
    while (<FILE>)
    {
        chomp;
		($keyword,$value)=split(/=/,$_);
        
        if ($keyword eq "hostname") {
            $my_hostname = $value;
        } elsif ($keyword eq "server_ip") {
            $server_ip = $value;
        } elsif ($keyword eq "server_port") {
            $server_port = $value;
        } elsif ($keyword eq "local_history") {
            $local_history = $value;
        } elsif ($keyword eq "history_separator") {
            $history_separator = $value;
        } elsif ($keyword eq "cpu_yellow") {
            $cpu_yellow = $value;
        } elsif ($keyword eq "cpu_red") {
            $cpu_red = $value;
        } elsif ($keyword eq "disk_yellow") {
            $disk_yellow = $value;
        } elsif ($keyword eq "disk_red") {
            $disk_red = $value;
        } elsif ($keyword eq "percent_physical_yellow") {
            $percent_physical_yellow = $value;
        } elsif ($keyword eq "percent_physical_red") {
            $percent_physical_red = $value;
        } elsif ($keyword eq "percent_actual_yellow") {
            $percent_actual_yellow = $value;
        } elsif ($keyword eq "percent_actual_red") {
            $percent_actual_red = $value;
        } elsif ($keyword eq "percent_swap_yellow") {
            $percent_swap_yellow = $value;
        } elsif ($keyword eq "percent_swap_red") {
            $percent_swap_red = $value;
        } elsif ($keyword eq "uptime_yellow") {
            $uptime_yellow = $value;
        }
    }
    close(FILE);
}

if (param("clear_history"))
{
    if (-e $PackageHistoryFile)
        { unlink($PackageHistoryFile); }
}

$param_hostname=param("hostname");
$param_server_ip=param("server_ip");
$param_server_port=param("server_port");
$param_local_history=param("local_history");
$param_history_separator=param("history_separator");
$param_cpu_yellow=param("cpu_yellow");
$param_cpu_red=param("cpu_red");
$param_disk_yellow=param("disk_yellow");
$param_disk_red=param("disk_red");
$param_percent_physical_yellow=param("percent_physical_yellow");
$param_percent_physical_red=param("percent_physical_red");
$param_percent_actual_yellow=param("percent_actual_yellow");
$param_percent_actual_red=param("percent_actual_red");
$param_percent_swap_yellow=param("percent_swap_yellow");
$param_percent_swap_red=param("percent_swap_red");
$param_uptime_yellow=param("uptime_yellow");

if ($param_hostname)
{
    $my_hostname = $param_hostname;
    if ($param_server_ip)
        { $server_ip = $param_server_ip; }
    if ($param_server_port)
        { $server_port = $param_server_port; }
    if ($param_local_history)
        { $local_history = $param_local_history; }
    else
        { $local_history = 0; }
    if ($param_history_separator)
        { $history_separator = $param_history_separator; }
    if ($param_cpu_yellow)
        { $cpu_yellow = $param_cpu_yellow; }
    if ($param_cpu_red)
        { $cpu_red = $param_cpu_red; }
    if ($param_disk_yellow)
        { $disk_yellow = $param_disk_yellow; }
    if ($param_disk_red)
        { $disk_red = $param_disk_red; }
    if ($param_percent_physical_yellow)
        { $percent_physical_yellow = $param_percent_physical_yellow; }
    if ($param_percent_physical_red)
        { $percent_physical_red = $param_percent_physical_red; }
    if ($param_percent_actual_yellow)
        { $percent_actual_yellow = $param_percent_actual_yellow; }
    if ($param_percent_actual_red)
        { $percent_actual_red = $param_percent_actual_red; }
    if ($param_percent_swap_yellow)
        { $percent_swap_yellow = $param_percent_swap_yellow; }
    if ($param_percent_swap_red)
        { $percent_swap_red = $param_percent_swap_red; }
    if ($param_uptime_yellow)
        { $uptime_yellow = $param_uptime_yellow; }
        
    $new_params = 1;
}

$error = "";
$err = 0;

if ($my_hostname eq "")
{
    $error = $lang_enter_hostname;
    $err = 1;
}
elsif ($server_port <= 0)
{
    $error = $lang_enter_server_port;
    $err = 1;
}

if (0 == $err)
{
    open(conf, "> $PackageAppConf") or die "Couldn't open $PackageAppConf for writing: $!\n";
    print conf "[$PackageName]\n";
    print conf "hostname=$my_hostname\n";
    print conf "server_ip=$server_ip\n";
    print conf "server_port=$server_port\n";
    print conf "local_history=$local_history\n";
    print conf "history_separator=$history_separator\n";
    print conf "cpu_yellow=$cpu_yellow\n";
    print conf "cpu_red=$cpu_red\n";
    print conf "disk_yellow=$disk_yellow\n";
    print conf "disk_red=$disk_red\n";
    print conf "percent_physical_yellow=$percent_physical_yellow\n";
    print conf "percent_physical_red=$percent_physical_red\n";
    print conf "percent_actual_yellow=$percent_actual_yellow\n";
    print conf "percent_actual_red=$percent_actual_red\n";
    print conf "percent_swap_yellow=$percent_swap_yellow\n";
    print conf "percent_swap_red=$percent_swap_red\n";
    print conf "uptime_yellow=$uptime_yellow\n";
    close(conf);

    if (1 == $new_params)
    {
        $result = `$PackageApp/$PackageFileName.pl param`;
    }
}

my $checked_local_history = "";

if (1 == $local_history)
{
    $checked_local_history = "checked=\"checked\"";
}

my $open_history_file = "";
if (-e $PackageHistoryFile)
{
    $open_history_file = <<EOF;
        &nbsp;&nbsp;&nbsp;
        <a href="$PackageFileName.csv" target="_blank" style="color:#000">$lang_history_link</a>
        &nbsp;&nbsp;&nbsp;
        <a href="index.cgi?clear_history=1" target="_self" style="color:#000">$lang_history_clear</a>
EOF
}

# HTML and JavaScript
#
print  <<EOF;
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />

<style type="text/css">
<!--

/*
  Example: http://woork.blogspot.ch/2008/06/clean-and-pure-css-form-design.html
*/

body {
font-family:"Lucida Grande", "Lucida Sans Unicode", Verdana, Arial, Helvetica, sans-serif;
font-size:12px;
}
p, h1, form, button {
    border:0; margin:0; padding:0;
}
.spacer {
    clear:both; height:1px;
}

.configuration_form {
margin:0 auto;
width:800px;
padding:14px;
}

#content {
border:solid 2px #b7ddf2;
background:#ebf4fb;
margin:4px;
}
#content h1 {
font-size:14px;
font-weight:bold;
margin-bottom:8px;
}
#content p{
font-size:11px;
color:#666666;
margin-bottom:20px;
padding-bottom:10px;
}
#content label{
clear:both;
display:block;
font-weight:bold;
text-align:right;
width:400px;
float:left;
margin-top:4px;
}
#content label:after {
    content:": ";
}

/***
#content .small{
color:#666666;
display:block;
font-size:11px;
font-weight:normal;
text-align:right;
width:400px;
}
***/
#content input.text{
float:left;
font-size:12px;
padding:4px 2px;
border:solid 1px #aacfe4;
width:200px;
margin:2px;
clear:right;
}
#content input.number{
float:left;
font-size:12px;
padding:4px 2px;
border:solid 1px #aacfe4;
width:40px;
margin:2px;
clear:right;
}
#content button{
clear:both;
display:block;
margin-right:10px;
width:125px;
height:31px;
background:#666666 no-repeat;
text-align:center;
line-height:31px;
color:#FFFFFF;
font-size:11px;
font-weight:bold;
}
#content input.checkbox{
float:left;
font-size:12px;
padding:4px 2px;
border:solid 1px #aacfe4;
margin:2px;
clear:right;
}
#content button:active{
background:#A0A0A0 no-repeat;
color:#FFA0A0;
}

#content input.paypal {
    vertical-align:middle;
}

#content number {
width:60px;
}

#content .bottom_line {
    border-bottom:solid 1px #b7ddf2;
}

-->
</style>

    <title>$lang_title</title>
    
</head>
<body style="margin: 1; padding: 1; text-align: left; font-family: Arial, Helvetica, sans-serif;">
    <div id="content">
        <div>
            <h1>$lang_title</h1>
            <p class="bottom_line">
            $lang_package_info <a href="$lang_url" target="_blank" style="color:#000">($lang_url)</a>
            $lang_package_center_info
            <form target="_blank" action="https://www.paypal.com/cgi-bin/webscr" method="post">
            <input type="hidden" name="cmd" value="_s-xclick">
            <input type="hidden" name="hosted_button_id" value="QWJJCNCJP2VGC">
            <input class="paypal" type="image" src="https://www.paypal.com/en_US/i/btn/btn_donate_SM.gif" border="0" name="submit" alt="Donate for multiotp using Paypal">
            $open_history_file
            </form>
            </p>

            <form id="configuration_form" name="configuration_form" method="post" action="index.cgi">
                <label for="local_history">$lang_local_history
                </label>
                <input class="checkbox" type="checkbox" name="local_history" id="local_history" $checked_local_history value="1" />

                <label for="history_separator">$lang_history_separator</label>
                <input type="text" name="history_separator" id="history_separator" value="$history_separator" class="number" />
                
                <label for="hostname">$lang_hostname
                <!-- <span class="small">$lang_enter_hostname</span> -->
                </label>
                <input class="text" type="text" name="hostname" id="hostname" value="$my_hostname" title="$lang_example my.hostname.fqdn" />
                                
                <label for="server_ip">$lang_server_ip</label>
                <input type="text" name="server_ip" id="server_ip" value="$server_ip" title="$lang_example my.xymon.server.ip" class="text" />
                                
                <label for="server_port">$lang_server_port</label>
                <input type="text" name="server_port" id="server_port" value="$server_port" title="$lang_example 1984" class="number" />

                <label for="uptime_yellow">$lang_uptime_yellow</label>
                <input type="text" name="uptime_yellow" id="uptime_yellow" value="$uptime_yellow" title="$lang_example 1800" class="number" />

                <label for="cpu_yellow">$lang_cpu_yellow</label>
                <input type="text" name="cpu_yellow" id="cpu_yellow" value="$cpu_yellow" title="$lang_example 6" class="number" />

                <label for="cpu_red">$lang_cpu_red</label>
                <input type="text" name="cpu_red" id="cpu_red" value="$cpu_red" title="$lang_example 12" class="number" />

                <label for="disk_yellow">$lang_disk_yellow</label>
                <input type="text" name="disk_yellow" id="disk_yellow" value="$disk_yellow" title="$lang_example 85" class="number" />

                <label for="disk_red">$lang_disk_red</label>
                <input type="text" name="disk_red" id="disk_red" value="$disk_red" title="$lang_example 95" class="number" />

                <label for="percent_physical_yellow">$lang_percent_physical_yellow</label>
                <input type="text" name="percent_physical_yellow" id="percent_physical_yellow" value="$percent_physical_yellow" title="$lang_example 99" class="number" />

                <label for="percent_physical_red">$lang_percent_physical_red</label>
                <input type="text" name="percent_physical_red" id="percent_physical_red" value="$percent_physical_red" title="$lang_example 100" class="number" />

                <label for="percent_actual_yellow">$lang_percent_actual_yellow</label>
                <input type="text" name="percent_actual_yellow" id="percent_actual_yellow" value="$percent_actual_yellow" title="$lang_example 70" class="number" />

                <label for="percent_actual_red">$lang_percent_actual_red</label>
                <input type="text" name="percent_actual_red" id="percent_actual_red" value="$percent_actual_red" title="$lang_example 90" class="number" />

                <label for="percent_swap_yellow">$lang_percent_swap_yellow</label>
                <input type="text" name="percent_swap_yellow" id="percent_swap_yellow" value="$percent_swap_yellow" title="$lang_example 80" class="number" />

                <label for="percent_swap_red">$lang_percent_swap_red</label>
                <input type="text" name="percent_swap_red" id="percent_swap_red" value="$percent_swap_red" title="$lang_example 90" class="number" />
                <div>
                    <button type="submit">$lang_update_configuration</button>
                    <div class="spacer"></div>
                </div>
            </form>
        </div>
        <div id="bottom">
            <div id="error">$error</div>
        </div>
    </div>
</body>
</html>
EOF
