#!/usr/bin/perl

######################################################################
#
# SynoMon - Xymon/Hobbit/BigBrother client for Synology DiskStation
# http://6co.ch/synomon/
#
# Donation are always welcome! Please check http://6co.ch/synomon/
# and you will find the magic button ;-)
#
# The Synology Monitoring tool is a Perl client for Xymon/Hobbit/BigBrother
# server that is tuned to work with Synology devices
#
#
# LICENCE
#
#   Copyright (c) 2012-2019, SysCo systemes de communication sa
#   SysCo (tm) is a trademark of SysCo systemes de communication sa
#   (http://www.sysco.ch/)
#   All rights reserved.
#
#   This file is part of the Synology Monitoring Tool
#
#   Synology Monitoring Tool is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public License as
#   published by the Free Software Foundation, either version 3 of the License,
#   or (at your option) any later version.
#
#   Synology Monitoring Tool is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Lesser General Public License for more details.
#
#   You should have received a copy of the GNU Lesser General Public
#   License along with Synology Monitoring Tool.
#   If not, see <http://www.gnu.org/licenses/>.
#
#
# @author: SysCo/al
# @since CreationDate: 2012-11-13
# @copyright (c) 2012-2019 by SysCo systemes de communication sa
# @version $LastChangedRevision: 1.5.3 $
my $PackageVersion="1.5.3";
# @version $LastChangedDate: 2019-11-23 $
# @version $LastChangedBy: SysCo/al & SysCo/lz $
# @link $HeadURL: synomon.pl $
# @link http://6co.ch/synomon/
# @link developer@sysco.ch
# Language: Perl (available for Synology devices as an additional package)
#
#
# Usage
#
#   Configuration page must be called after the installation
#
#
# Configuration file format (/etc/synomon.ini)
#
#   [SynoMon]
#   hostname=hostname.fqdn
#   server_ip=ip.addr.of.server
#   server_port=server.port
#   local_history=1
#   history_separator=;
#   cpu_yellow=6
#   cpu_red=12
#   disk_yellow=85
#   disk_red=95
#   percent_physical_yellow=99
#   percent_physical_red=100
#   percent_actual_yellow=70
#   percent_actual_red=90
#   percent_swap_yellow=80
#   percent_swap_red=90
#   uptime_yellow=1800
#
#
# Examples
#
#  Usage is straightforward
#
#
# External files created
#
#   /etc/synomon.ini (which is the configuration file)
#
#
# External file modified
#
#   /etc/crontab (in order to schedule the monitoring every 5 minutes)
#
#
# Special issues
#
#   If you need specific developments concerning this monitoring tool
#   do not hesistate to contact us per email at developer@sysco.ch.
#
#
# Other related ressources
#
#   raid information extraction from
#   Michal Ludvig <michal\@logix.cz> - http://www.logix.cz/michal/devel/nagios
#   Diego d'Ambra <da\@panorama9.com> - http://panorama9.com
#
#
#
# Users feedbacks and comments
#
# 2019-11-23 1.5.3 SysCo/al
#   As proposed by Henk Groenewoud from SanTec, patch to avoid xymond crashing after upgrade to XyMon 4.3.30
#
# 2019-11-08 1.5.1 SysCo/al
#   As proposed by Ernst Mikkelsen from TRIFORK, detailed information starts now with the OS
#
# 2016-04-18 SysCo/lz
#   Support for additional devices and other enhancements
#
# 2012-11-28 SysCo/al
#   As proposed by Ltek, adding a local history file option for history and debbuging purposes
#
# 2012-11-13 SysCo/al
#   Well, as requested by some users, the monitoring tool is now a package, enjoy !
#
#
# TODOs
#
#   ExtJS version of the UI
#   Adding Nagios support (NSCA Perl Client, see http://exchange.nagios.org/directory/Addons/Passive-Checks/NSCA-Perl-Client/details)
#
#
# Change Log
#
#   2019-11-23 1.5.3  SysCo/al Fix to avoid crashing xymond (after upgrade to XyMon 4.3.30, thanks to Henk.
#                              Internal IP address(es) added in the general info header
#   2019-11-08 1.5.1  SysCo/al Detailed information reorganized to put Linux version at the first line
#   2017-01-05 1.4.8  SysCo/al Updated distribution
#   2017-01-04 1.4.7  SysCo/al Added CGI.pm and CGI/Util.pm (removed in the current Perl distribution)
#   2017-01-04 1.4.6  SysCo/al Remove Perl switch usage (also for index.cgi)
#   2017-01-04 1.4.5  SysCo/al Remove switch usage (not supported in the last Perl package from Synology)
#   2016-04-18 1.4.4  SysCo/al CPU model better detected on all plateforms
#   2016-04-18 1.4.3  SysCo/al CPU model better detected on all plateforms
#                              Degree () is now written as an html entity &deg;
#                              procs alarm with better layout
#                              procs alarm with more details on current processes
#   2016-04-07 1.4.2  SysCo/al Package is signed with our private key
#                              Public key is automatically pushed from http://synology.sysco.ch
#                              Package can bu updated automatically
#                              All alarms returns now SynoMon version, NAS type, firmware, processor, MAC address
#                              cpu alarm with more details on current processes and in an ordered list
#                              disk alarm with no red warning anymore for usb drives
#                              netstat alarm includes IP information (ifconfig)
#                              temperature alarm with system and disks temperatures
#                              ups alarm with better yellow and red states handling
#   2016-04-02 1.4.1  SysCo/al Better DSM 6 support with more general info
#                              procs alarm with more details on current processes
#                              raid alarm with disks SMART states, serial numbers (was lost in DSM5 if more than one drive)
#   2016-04-02 1.4.0  SysCo/al Better DSM 6 support (enhanced procs info)
#   2016-03-31 1.3.3  SysCo/al DSM 6 support (CPU was not monitored anymore)
#   2014-02-24 1.3.0  SysCo/lz Now all volumes are monitored
#   2013-09-19 1.2.9  SysCo/al Admin only seems not working everywhere, removed
#   2013-03-05 1.2.8  SysCo/al Disk usage reformatted to handle correctly history graphs
#   2013-03-05 1.2.7  SysCo/al Adding Perl dependency only if DSM 4.2 or higher (otherwise 1.2.5)
#   2013-03-05 1.2.6  SysCo/al Adding Perl dependency
#   2013-03-05 1.2.5  SysCo/al Adding the package version in the uptime information
#   2013-03-05 1.2.4  SysCo/al No alert for md0 and md1 partitions if not all disks are installed in the bay
#   2013-01-13 1.2.3  SysCo/al Total swap memory and usage in percent correctly calculated
#   2012-12-15 1.2.2  SysCo/al Enhanced package scripts
#   2012-12-15 1.2.1  SysCo/al Sending UPS test result only if UPS status is known
#   2012-12-15 1.2.0  SysCo/al Adding UPS support
#   2012-12-01 1.1.2  SysCo/al More options in the UI
#   2012-11-28 1.1.1  SysCo/al Updated UI
#   2012-11-28 1.1.0  SysCo/al Adding local history file option for history and debbuging purposes
#                              Better start/stop of the cron daemon in the start-stop-status file
#                              More generic script (for other packages)
#   2011-11-18 1.0.2  SysCo/al Integrated in synology.sysco.ch package center
#   2011-11-14 1.0.1  SysCo/al Launch a monitoring after configuration update
#   2011-11-14 1.0.0  SysCo/al Perl configuration UI
#   2011-11-14 0.9.2  SysCo/al Installation issues
#   2012-11-13 0.9.1  SysCo/al Initial implementation
#
#####################################################################/


use strict;
use IO::Socket;
use POSIX;

sub trim($);

my $PackageName="SynoMon";
my $PackageNiceName="Synology Monitoring Tool";
my $PackageFileName="synomon";
my $PackageLink="http://6co.ch/synomon/";
my $PackageApp="/var/packages/$PackageName/target/app";
my $PackageConf="/etc/$PackageFileName.ini";
my $PackageLog=$PackageApp."/debug.log";
my $VarLogFolder="/var/log";
my $PackageHistoryFile="$VarLogFolder/$PackageFileName.csv";
my $PackageHistoryFileLink="$PackageApp/$PackageFileName.csv";

my $my_hostname = read_file('/proc/sys/kernel/hostname');
my $server_ip = "";
my $server_port = 1984;

my $local_history = "0";
my $history_separator = ";";

my $cpu_yellow = 6.0;
my $cpu_red = 12.0;

my $disk_yellow = 85;
my $disk_red = 95;

my $percent_physical_yellow = 99;
my $percent_physical_red = 100;
my $percent_actual_yellow = 70;
my $percent_actual_red = 90;
my $percent_swap_yellow = 80;
my $percent_swap_red = 90;

my $uptime_yellow = 30*60;

my %hash;


sub trim($)
{
    my $string = shift;
    $string =~ s/^\s+//;
    $string =~ s/\s+$//;
    return $string;
}

sub read_file
{
    my $file = shift;

    my $ret_file;
    open(FILE, "<$file") || return "unknown";
    while (<FILE>)
    {
        chomp;
        $ret_file .= $_;
    }
    close(FILE);

    #return content of file
    return $ret_file;
}


sub read_original_file
{
    my $file = shift;

    my $ret_file;
    open(FILE, "<$file") || return "unknown";
    while (<FILE>)
    {
        $ret_file .= $_;
    }
    close(FILE);

    #return content of file
    return $ret_file;
}


sub locate_tool
{
    my $tool = shift;

    #find tool
    return $1 if ( `which $tool` =~ /(.+)/ );
}


#####################################################################
#
# Name: convert_time
# Short description: Convert elapsed time from seconds to full info
#
# Creation 2012-11-13
# Update 2010-11-13
# @package synomon
# @version 1.0.0
# @author SysCo/al
#
# @param   int     $time  Elapsed time in seconds
# @return  string         Elapsed time in day(s), hour(s), minute(s), second(s)
#
#####################################################################
sub convert_time
{
    my $time = shift;
    my $days = int($time / 86400);
    $time -= ($days * 86400);
    my $hours = int($time / 3600);
    $time -= ($hours * 3600);
    my $minutes = int($time / 60);
    my $seconds = $time % 60;

    $days = $days < 1 ? '' : $days .' days ';
    $hours = $hours < 1 ? '' : $hours .' hours ';
    $minutes = $minutes < 1 ? '' : $minutes . ' minutes ';
    $time = $days . $hours . $minutes . $seconds . ' seconds';
    return $time;
}


sub convert_time_to_days
{
    my $time = shift;
    my $days = int($time / 86400);

    $days = $days < 1 ? '0 day ' : $days .' days';
    return $days;
}


#send_info('server_ip',server_port,'hostname','test_name','color','detailed_text','lifetime');
# color: green / yellow / red
sub send_info
{
    my ($server_ip,$server_port,$hostname, $test, $color, $message, $lifetime) = @_;
    my ($debug,$invalid,$start,$sock,$prev);
    # Timeout in seconds
    my $timeout = 10;
    my $data = "";
    my $debug = 0;
    if (!defined($lifetime))
    {
        $lifetime = 5*60;
    }

    my $bb_hostname = $hostname;
    $bb_hostname =~ s/\./,/;

    $data .= "status+$lifetime $bb_hostname.$test $color $message";

    $sock = new IO::Socket::INET (PeerAddr => $server_ip,
                                  PeerPort => $server_port,
                                  Timeout  => $timeout)
                                 or die("Unable to open $server_ip:$server_port");
    print "State of $test sent to $server_ip:$server_port\n";
    my $bytes = 0;
    die "Failed\n"
        if send($sock,$data,0) != length($data);
    $bytes += length($data);
    print STDERR "$data\n"
        if $debug;
}


#get_uptime(yellow_limit_in_seconds);
sub get_uptime
{
    my ($yellow_limit_in_seconds) = @_;
    if (!defined($yellow_limit_in_seconds))
    {
        $yellow_limit_in_seconds = 30*60;
    }
    my $uptime = $1
        if (read_file('/proc/uptime') =~ /^([^\.]+)/);

    my $uptime_color = "green";

    # up to red limit: RED
        #case { $_[0] < ($red_limit_in_seconds) }
    # up to yellow limit: yellow
    if ($uptime < ($yellow_limit_in_seconds)) {
        $uptime_color = 'yellow';
    } else {
        $uptime_color = 'green';
    }

    return ($uptime, $uptime_color);
}


#get_memory(yellow_percent_physical,red_percent_physical,yellow_percent_actual,red_percent_actual,yellow_percent_swap,red_percent_swap);
sub get_memory
{
    my ($yellow_percent_physical, $red_percent_physical, $yellow_percent_actual, $red_percent_actual, $yellow_percent_swap, $red_percent_swap) = @_;
    if (!defined($yellow_percent_physical))
    {
        $yellow_percent_physical = 80;
    }
    if (!defined($red_percent_physical))
    {
        $red_percent_physical = 90;
    }
    if (!defined($yellow_percent_actual))
    {
        $yellow_percent_actual = 80;
    }
    if (!defined($red_percent_actual))
    {
        $red_percent_actual = 90;
    }
    if (!defined($yellow_percent_swap))
    {
        $yellow_percent_swap = 80;
    }
    if (!defined($red_percent_swap))
    {
        $red_percent_swap = 90;
    }

    my $memory = "\tMemory\t\tUsed\tTotal\tPercentage\n";

    my $MemFree = `/bin/cat /proc/meminfo | /bin/grep "^MemFree" | /bin/sed -e "s/.*://g" -e "s/ k.*//g" -e "s/ *//g"`;
    my $MemTotal = `/bin/cat /proc/meminfo | /bin/grep "^MemTotal" | /bin/sed -e "s/.*://g" -e "s/ k.*//g" -e "s/ *//g"`;
    my $Buffers = `/bin/cat /proc/meminfo | /bin/grep "^Buffers" | /bin/sed -e "s/.*://g" -e "s/ k.*//g" -e "s/ *//g"`;
    my $Cached = `/bin/cat /proc/meminfo | /bin/grep "^Cached" | /bin/sed -e "s/.*://g" -e "s/ k.*//g" -e "s/ *//g"`;
    my $SwapFree = `/bin/cat /proc/meminfo | /bin/grep "^SwapFree" | /bin/sed -e "s/.*://g" -e "s/ k.*//g" -e "s/ *//g"`;
    my $SwapTotal = `/bin/cat /proc/meminfo | /bin/grep "^SwapTotal" | /bin/sed -e "s/.*://g" -e "s/ k.*//g" -e "s/ *//g"`;

    my $PhysUsed = int(($MemTotal - $MemFree) / 1024);
    my $ActuUsed = int(($MemTotal - $MemFree - $Buffers - $Cached) / 1024);
    my $SwapUsed = int(($SwapTotal - $SwapFree) / 1024);

    my $MegaMemTotal = int($MemTotal / 1024);
    my $MegaSwapTotal = int($SwapTotal / 1024);

    my $PhysColor = "green";
    my $PhysRatio = $MemTotal / 100 / 1024;
    my $PhysPercent = int($PhysUsed / $PhysRatio);
    if ($PhysPercent >= $yellow_percent_physical) { $PhysColor = "yellow" } ;
    if ($PhysPercent >= $red_percent_physical) { $PhysColor = "red" } ;
    $memory .= "&".trim($PhysColor)."\tPhysical\t".trim($PhysUsed)."M\t".trim($MegaMemTotal)."M\t".trim($PhysPercent)."%\n";

    my $ActuColor = "green";
    my $ActuRatio = $MemTotal / 100 / 1024;
    my $ActuPercent = int($ActuUsed / $ActuRatio);
    if ($ActuPercent >= $yellow_percent_actual) { $ActuColor = "yellow" };
    if ($ActuPercent >= $red_percent_actual) { $ActuColor = "red" };
    $memory .= "&".trim($ActuColor)."\tActual\t\t".trim($ActuUsed)."M\t".trim($MegaMemTotal)."M\t".trim($ActuPercent)."%\n";

    my $SwapColor = "green";
    my $SwapRatio = $SwapTotal / 100 / 1024;
    my $SwapPercent = int($SwapUsed / $SwapRatio);
    if ($SwapPercent >= $yellow_percent_swap) { $SwapColor = "yellow" };
    if ($SwapPercent >= $red_percent_swap) { $SwapColor = "red" };
    $memory .= "&".trim($SwapColor)."\tSwap\t\t".trim($SwapUsed)."M\t".trim($MegaSwapTotal)."M\t".trim($SwapPercent)."%\n";

    my $memory_color = "green";

    if (($PhysColor eq "yellow") || ($ActuColor eq "yellow") || ($SwapColor eq "yellow")) { $memory_color = "yellow" };

    if (($PhysColor eq "red") || ($ActuColor eq "red") || ($SwapColor eq "red")) { $memory_color = "red" };

    return ($memory, $memory_color, $PhysUsed, $ActuUsed, $SwapUsed);
}


#get_cpu(yellow_load,red_load);
sub get_cpu
{
    my ($yellow_load,$red_load) = @_;
    if (!defined($yellow_load))
    {
        $yellow_load = 6.0;
    }
    if (!defined($red_load))
    {
        $red_load = 12.0;
    }
    my $cpu;
    my $cpu_load = 0;

#    open( CMD, "sudo /bin/top -n1 |" );
#    while (<CMD>)
#    {
#        $cpu .= $_;
#        #CPU:  0.0% usr  9.5% sys  0.0% nic 90.4% idle  0.0% io  0.0% irq  0.0% sirq
#        #Load average: 0.06 0.04 0.07 2/186 24807
#        #top - 20:19:02 up 7 days,  6:49,  1 user,  load average: 0.82, 0.85, 0.95
#        if ($_=~/.*Load average: (.*?)[, ]+(.*?)[, ]+(.*?)$/i)
#        {
#            $cpu_load = $2;
#        }
#    }
#    close(CMD);

    open( CMD, "uptime |" );
    while (<CMD>)
    {
        $cpu .= $_;

        # 07:00:39 up 38 days, 21:57, load average: 1.26, 1.36, 1.49
        if ($_=~/.*Load average: (.*?)[, ]+(.*?)[, ]+(.*?)$/i)
        {
            $cpu_load = $2;
        }
    }
    close(CMD);

    $cpu =~ s/.\[.m//mg;

    $cpu .= "\nAlarm is on $yellow_load / $red_load\n";

    my $cpu_color = "green";
    if ($cpu_load < $yellow_load) {
        # < 6 green
        $cpu_color = "green";
    } elsif ($cpu_load < $red_load) {
        # < 12 yellow
        $cpu_color = "yellow";
    } else {
        $cpu_color = "red";
    }

    return ($cpu, $cpu_color, $cpu_load);
    }

#get_disk(yellow_percent,red_percent);
sub get_disk
{
    my ($yellow_percent,$red_percent) = @_;
    if (!defined($yellow_percent))
    {
        $yellow_percent = 85;
    }
    if (!defined($red_percent))
    {
        $red_percent = 95;
    }
    my $disk;
    my $disk_total = 0;
    my $disk_used_size = 0;
    my $disk_free = 0;
    my $disk_used = 0;
    my $disk_volume = "";

    my $disk_color = "green";

    open( CMD, "/bin/df -h |" );
    while (<CMD>)
    {
        chomp;
        $disk .= $_."\n";
        if (/^.*\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)%\s+([^\s]+).*$/)
        {
            # $disk_total = $1; $disk_used_size = $2; $disk_free = $3; $disk_used = $4; $disk_volume = $5;
            if ($4 > $disk_used) {
                if ($4 < $yellow_percent) {
                    # < 85 green
                    $disk_color = "green";
                    $disk_used = $4;
                } elsif ($4 < $red_percent) {
                    # < 95 yellow
                    $disk_color = "yellow";
                    $disk_used = $4;
                } else {
                    if ( $5 =~ /.*USB.*/ ) {
                        # USB /volumeUSB1/usbshare
                        if ($disk_color ne "red") {
                            $disk_color = "red";
                        }
                    } else {
                        $disk_color = "red";
                        $disk_used = $4;
                    }
                }
            }
        }
    }
    close(CMD);

    return ($disk, $disk_color, $disk_used);
}


#ALWAYS green
sub get_who
{
    my $who;
    my $who_count = 0;
    open( CMD, "who |" );
    while (<CMD>)
    {
        $who .= $_;
        #USER       TTY      IDLE      TIME            HOST
        #root       pts/0    00:00     Oct 21 01:02:12 192.168.21.5
        if ($_=!/^USER(.*?)$/)
        {
            $who_count++;
        }
    }
    close(CMD);

    my $who_color = "green";

    return ($who, $who_color, $who_count);
}


#ALWAYS green
sub get_procs
{
    my $procs;
    my $procs_count = 0;
    my $enhanced = 0;
    my $test = '';
    open( CMD, "ps |" );
    while (<CMD>)
    {
        $procs .= $_;
        $test = $_;
        #  PID USER       VSZ STAT COMMAND
        #    1 root      2536 S    init
        if ($_=!/^(.*?)PID(.*?)$/)
        {
            $procs_count++;
        }
        if ($test=~/.*TIME.*CMD.*$/)
        {
            $enhanced = 1;
        }
    }
    close(CMD);

    if ($enhanced > 0) {
        $procs_count = 0;
        $procs = "";
        open( CMD, "ps -aux --sort=-pcpu |" );
        while (<CMD>)
        {
            $procs .= $_;
            if ($_=!/^(.*?)PID(.*?)$/)
            {
                $procs_count++;
            }
        }
        close(CMD);
    }

    $procs =~ s/</&lt;/mg;
    $procs =~ s/>/&gt;/mg;

    my $procs_color = "green";

    return ($procs, $procs_color, $procs_count);
}


#ALWAYS green
sub get_netstat
{
    my $netstat;
    open( CMD, "netstat -an |" );
    while (<CMD>) {
        $netstat .= $_;
    }
    close(CMD);

    $netstat .= "\n";

    open( CMD, "/sbin/ifconfig |" );
    while (<CMD>) {
        $netstat .= $_;
    }
    close(CMD);

    my $netstat_color = "green";

    return ($netstat, $netstat_color);
}


sub get_raid_state
{
    my $raid_state = '';
    my $raid_state_color = 'green';

    my $mdstat_file = "/proc/mdstat";
    my $disk_device = "all";

    my %ERRORS = ( 'OK' => 0, 'WARNING' => 1, 'CRITICAL' => 2, 'UNKNOWN' => 3, 'DEPENDENT' => 4 );

    #this is a global return value - set to the worst result we get overall
    my $retval = 0;


    #info about physical disks
    my %hdd_info;
    my $hhd_cmd = locate_tool('/usr/syno/bin/syno_hdd_util');
    if ( defined($hhd_cmd) ) {

        #get info about hdd
        open( CMD, "$hhd_cmd --ssd_detect |" );
        my $l;
        while (<CMD>) {
            chomp;
            next if ( !/^([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+(\/dev\/\wd[^0-9]*?)\s+([^\s]+)/ );
            $hdd_info{$4} = {
                'model'    => $1,
                'firmware' => $2,
                'serial'   => $3,
                'is_ssd'   => $5,
            };
        }
        close(CMD);
    }

    #info about model, serial, hostname, firmware, number of disks and uptime
    my $model    = '/proc/sys/kernel/syno_hw_version';
    my $serial   = '/proc/sys/kernel/syno_serial';
    my $hostname = '/proc/sys/kernel/hostname';
    my $firmware = "$1.$2-$3"
      if ( read_file('/etc.defaults/VERSION') =~ /majorversion="(\d+)"minorversion="(\d+)".+buildnumber="(\d+)/ );
    my $uptime = $1 if ( read_file('/proc/uptime') =~ /^([^\.]+)/ );
    $raid_state .= "OK - SYSTEM, model: "
      . read_file($model)
      . ", ser: "
      . read_file($serial)
      . ", hn: "
      . read_file($hostname)
      . ", fw: $firmware, disks: "
      . scalar( keys %hdd_info )
      . ", up: $uptime\n";

    #map logical volumes to devices
    my (%dev_info);
    my $df_cmd = locate_tool('df');
    if ( defined($df_cmd) ) {

        #size and free space of each logical volume
        open( CMD, "$df_cmd |" );
        my $l;
        while (<CMD>) {
            chomp;
            $l = $_;

            #df may split filesystem status into 2 lines and Busybox doesn't support cmd-line option -P (join if needed)
            $l .= ' ' . <CMD> if ( $l =~ /^[^\s]+$/ );

            #split line
            if ( $l =~ /^([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+[^\s]+\s+([^\s]+)/ ) {
                my $fs        = $1;
                my $size      = $2;
                my $used      = $3;
                my $available = $4;
                my $mount     = $5;
                my $vol_num   = $1 if ( $mount =~ /(\d+)$/ );
                next unless ($vol_num);

                #single or multi bay NAS?
                my $dev;
                if ( -e '/dev/md' . ( $vol_num + 1 ) ) {
                    $dev = '/dev/md' . ( $vol_num + 1 );
                }
                elsif ( -e '/dev/hd' . ( $vol_num + 2 ) ) {
                    $dev = '/dev/hd' . ( $vol_num + 2 );
                }

                #store device info (only map first found)
                if ( !exists( $dev_info{$dev} ) ) {
                    $dev_info{$dev} = {
                        'available' => $available,
                        'size'      => $size,
                        'used'      => $used,
                        'mount'     => $mount,
                    };
                }
            }
        }
        close(CMD);

    }

    #open file (mdstat)
    my ( %active_devs, %failed_devs, %spare_devs, %drives );
    open FILE, "< $mdstat_file" or die "Can't open $mdstat_file : $!";
    while (<FILE>) {
        next if !/^(md\d+)+\s*:\s*\w+\s*(raid\d+)/;
        next if $disk_device ne "all" and $disk_device ne $1;
        my $dev  = $1;
        my $raid = $2;

        my @array = split(/ /);
        for $_ (@array) {
            next if !/([a-zA-Z]+)\d+\[\d+\](\(.\))*/;
            if ( $2 eq "(F)" ) {
                $failed_devs{$dev} .= "$1,";
            } elsif ( $2 eq "(S)" ) {
                $spare_devs{$dev} .= "$1,";
            } else {
                $active_devs{$dev} .= "$1,";
            }

            #remember drive, since we need detailed info about each physical disk
            $drives{$1} = 1;

        }
        if ( !defined( $active_devs{$dev} ) ) {
            $active_devs{$dev} = "none";
        } else {
            $active_devs{$dev} =~ s/,$//;
        }
        if ( !defined( $spare_devs{$dev} ) ) {
            $spare_devs{$dev} = "none";
        } else {
            $spare_devs{$dev} =~ s/,$//;
        }
        if ( !defined( $failed_devs{$dev} ) ) {
            $failed_devs{$dev} = "none";
        } else {
            $failed_devs{$dev} =~ s/,$//;
        }

        $_ = <FILE>;
        /\[(\d+)\/(\d+)\]\s+\[(.*)\]$/;
        my $devs_total = $1;
        my $devs_up    = $2;
        my $stat       = $3;
        my $result     = "OK";
        if ((($devs_total > $devs_up) and ($dev ne "md0") and ($dev ne "md1")) or $failed_devs{$dev} ne "none" ) {
            $result = "CRITICAL";
            $retval = $ERRORS{"CRITICAL"};
        }

        #print result
        $raid_state .= "$result - RAID, $dev $raid [$stat] $devs_up/$devs_total "
          . "(A:$active_devs{$dev} F:$failed_devs{$dev} S:$spare_devs{$dev})"
          . (
            exists( $dev_info{ '/dev/' . $dev } )
            ? ", size: $dev_info{'/dev/'.$dev}{'size'}, free: $dev_info{'/dev/'.$dev}{'available'}, name: $dev_info{'/dev/'.$dev}{'mount'}"
            : ''
          ) . "\n";
    }
    close FILE;

    my $hdd_temperature = "";

    #health of each physical disk
    my %seen;
    my $smartctl_cmd = locate_tool('smartctl');
    if ( defined("$smartctl_cmd") ) {
        foreach my $d ( sort {$a cmp $b} keys %drives ) {
            #map found drive to physical disk
            #my $pd = $1 if ( $d =~ /\wd(\w)/ );
            #my $drive = '/dev/hd' . $pd;
            #print "DEBUG Disk: $d $pd\n";

            #NEW drive detection
            my $drive = "/dev/".$1."d".$2 if ( $d =~ /^(\w)d([^0-9]*)/ );

            next if ( exists( $seen{$drive} ) );
            $seen{$drive} = 1;

            #get drive info
            my ( $vendor, $model, $size, $temperature );
            my $temperature_text = '';
            open( CMD, "$smartctl_cmd -d sat -i -A $drive |" ) || next;
            while (<CMD>) {
                $model = $1 if (/^Device Model:\s+(.+)/);
                #$size  = $1 if (/^User Capacity:\s+([\d\.,]+)/);
                #$vendor = $1 if (/^Vendor:\s+(.+)/);
                #$model = $1 if (/^Product:\s+(.+)/);
                $size  = $1 if (/^User Capacity:\s+([\d\.,]+)/);
                $temperature = $1 if (/Temperature_Celsius\s+[^\s]*\s+[^\s]*\s+[^\s]*\s+[^\s]*\s+[^\s]*\s+[^\s]*\s+[^\s]*\s+([\d\.,]+)/);
            }
            if ( defined($temperature) ) {
                $temperature_text = ", $temperature&deg;C";
                $hdd_temperature.= "&green\t".substr($drive."          ",0,10)."\t$temperature\t".int (($temperature * 9 / 5) + 32)."\n";
            }
            close(CMD);
            next unless ( defined($model) );
            $size =~ s/(,|\.)//g;

            #get smartctrl status
            my $health = 'CRITICAL';
            open( CMD, "$smartctl_cmd -d sat -H $drive |" ) || next;
            while (<CMD>) {
                $health = 'OK' if (/test result:\s+passed/i);
                $health = 'OK' if (/Status:\s+OK/i);
            }
            close(CMD);

            #print result
            $raid_state .= "$health - HD $drive, model: $model, size: $size, serial: "
              . ( exists( $hdd_info{ $drive } ) ? $hdd_info{ $drive }{'serial'} : 'unknown' )
              . "$temperature_text\n";
            $retval = $ERRORS{"CRITICAL"} unless ( $health eq 'OK' );
        }
    }

    my @temp_array = split /(?<=\n)/, $hdd_temperature;
    @temp_array = sort @temp_array;
    $hdd_temperature = join("", @temp_array);

    if ($retval == 0) {
        $raid_state_color = 'green';
    } elsif ($retval == 1) {
        $raid_state_color = 'yellow';
    } elsif ($retval == 2) {
        $raid_state_color = 'red';
    } else {
        $raid_state_color = 'red';
    }

    return ($raid_state, $raid_state_color, $hdd_temperature);
}


sub get_info
{
    my $model           = read_file('/proc/sys/kernel/syno_hw_version');
    my $serial          = read_file('/proc/sys/kernel/syno_serial');
    my $device_hostname = read_file('/proc/sys/kernel/hostname');
    my $linux_version   = read_file('/proc/version ');
    my $firmware        = "$1.$2-$3"
        if (read_file('/etc.defaults/VERSION') =~ /majorversion="(\d+)"minorversion="(\d+)".+buildnumber="(\d+)/ );
    my $system_temperature = "-";
    my $sys_temp_file = "";
    my $cpu_model = "";

    # my $proc_speed = read_file('/proc/sys/kernel/syno_CPU_info_clock');
    $cpu_model = "$1"
        if (read_original_file('/proc/cpuinfo') =~ /Processor.*?:[ ]*?(.*)?[\f\n\r]/ );
    if ($cpu_model == "") {
        $cpu_model = "$1"
            if (read_original_file('/proc/cpuinfo') =~ /model name.*?:[ ]*?(.*)?[\f\n\r]/ );
    }

    my @sys_temp_files = ("/sys/class/hwmon/hwmon0/device/temp1_input", "/sys/class/hwmon/hwmon0/device/temp2_input", "/sys/devices/virtual/hwmon/hwmon0/temp1_input", "/sys/devices/virtual/hwmon/hwmon0/temp2_input", "/sys/bus/platform/devices/axp-temp.0/temp1_input", "/sys/bus/platform/devices/axp-temp.0/temp2_input", "/sys/bus/platform/devices/coretemp.0/temp1_input", "/sys/bus/platform/devices/coretemp.0/temp2_input");
    foreach $sys_temp_file (@sys_temp_files) {
        if (-e $sys_temp_file) {
            $system_temperature = read_file($sys_temp_file);
            if ($system_temperature > 1000){
                $system_temperature = ($system_temperature / 1000);
            }
            last;
        }
    }

    my $netif_num = read_file('/proc/sys/kernel/syno_internal_netif_num');
    my $mac_addresses = read_file('/proc/sys/kernel/syno_mac_address1');
    for (my $i = 2; $i <= $netif_num; $i++) {
        $mac_addresses .= ", ".read_file('/proc/sys/kernel/syno_mac_address'.$i);
    }


    my $ip_addresses = "";
    my $one_address = "";
    open( CMD, "/sbin/ifconfig | grep Mask | awk '{print \$2}'| cut -f2 -d: |" );
    while (<CMD>)
    {
        $one_address = trim($_);
        if ($one_address ne "127.0.0.1") {
            if ($ip_addresses ne "") {
                $ip_addresses .= ", ";
            }
            $ip_addresses .= $one_address;
        }
    }
    close(CMD);

    return ($model, $serial, $device_hostname, $firmware, $linux_version, $cpu_model, $system_temperature, $mac_addresses, $ip_addresses);
}


sub get_ups
{
    my $ups;
    my $ups_full;

    # OK = 0, WARNING = 1, CRITICAL = 2, UNKNOWN = 3
    my $ups_error;

    my $ups_color;
    my $ups_state;
    my $ups_state_separator;
    my $ups_load = 0;
    my $ups_runtime = 0;
    my $info_line;
    my $info_key;
    my $name;
    my $value;
    my %info_hash;

    my $ups_error = 0;

    my $upsc_cmd = locate_tool('upsc');
    if ( defined($upsc_cmd) ) {
        open( CMD, "$upsc_cmd ups |" );
        while (<CMD>) {
            $ups_full .= $_;
            $info_line=$_;
            chop ($info_line);          # Get rid of the trailling \n
            $info_line =~ s/^\s*//;     # Remove spaces at the start of the line
            $info_line =~ s/\s*$//;     # Remove spaces at the end of the line
            # Ignore lines starting with # and blank lines
            if ( ($info_line !~ /^#/) && ($info_line ne "") ) {
                ($name, $value) = split (/:/, $info_line); # Split each line into name value pairs
                $name =~ s/^\s*//; $name =~ s/\s*$//;
                $value =~ s/^\s*//; $value =~ s/\s*$//;
                $info_hash{$name} = $value; # Create a hash of the name value pairs
            }
        }

        #
        #foreach $info_key (keys %info_hash)
        #{
        #    print "$info_key = $info_hash{$info_key}\n";
        #}
        #

        close(CMD);

        $ups_error = 0;

        if (defined($info_hash{'ups.mfr'})) {
            $ups.= $info_hash{'ups.mfr'}." ";
        }
        if (defined($info_hash{'ups.model'})) {
            $ups.= $info_hash{'ups.model'};
        }
        if (defined($info_hash{'ups.serial'})) {
            $ups.= " (serial number: ".$info_hash{'ups.serial'}.")";
        }

        my @ups_status = split(' ', $info_hash{'ups.status'});
        foreach $value (@ups_status) {
            if ($value eq "ALARM") {
                $ups_error = 2;
            } elsif ($value eq "OL") {
                $ups_state.= $ups_state_separator."ON LINE";
                $ups_state_separator = ", ";
            } elsif ($value eq "OB") {
                $ups_state.= $ups_state_separator."ON BATTERY";
                $ups_state_separator = ", ";
                $ups_error = 2;
            } elsif ($value eq "CHRG") {
                $ups_state.= $ups_state_separator."CHARGING";
                $ups_state_separator = ", ";
            } elsif ($value eq "DISCHRG") {
                $ups_state.= $ups_state_separator."DISCHARGING";
                $ups_state_separator = ", ";
            } elsif ($value eq "LB") {
                $ups_state.= $ups_state_separator."LOW BATTERY";
                $ups_state_separator = ", ";
                if ($ups_error < 2) {
                    $ups_error = 1;
                }
            } elsif ($value eq "RB") {
                $ups_state.= $ups_state_separator."REPLACE BATTERY";
                $ups_state_separator = ", ";
                $ups_error = 2;
            } elsif ($value eq "FSD") {
                $ups_state.= $ups_state_separator."FORCED SHUTDOWN STATE";
                $ups_state_separator = ", ";
                if ($ups_error < 2) {
                    $ups_error = 1;
                }
            }
        }

        if (index($info_hash{'ups.status'}, 'ALARM') != -1) {
            $ups_error = 2;
            $ups_state.= $info_hash{'ups.alarm'}.$ups_state_separator.$ups_state;
        }

        $ups.="\nStatus: ".$ups_state;

        if (defined($info_hash{'ups.load'})) {
            $ups_load = $info_hash{'ups.load'};
            $ups.= "\nLoad: $ups_load%";
        }
        if (defined($info_hash{'battery.runtime'})) {
            $ups_runtime = $info_hash{'battery.runtime'};
            $ups.= "\nEstimated runtime: $ups_runtime seconds";
        }

        $ups.="\n\n".$ups_full;

    }

    if (!defined($info_hash{'ups.status'})) {
        $ups_error = 3;
    }

    if ($ups_error == 0) {
        $ups_color = 'green';
    } elsif ($ups_error == 1) {
        $ups_color = 'yellow';
    } elsif ($ups_error == 2) {
        $ups_color = 'red';
    } elsif ($ups_error == 3) {
        $ups_color = 'clear';
    } else {
        $ups_color = 'red';
    }

    return ($ups, $ups_color, $ups_state, $ups_load, $ups_runtime);
}


########
# MAIN #
########

my $datetime = POSIX::strftime("%Y-%m-%d %H:%M:%S", localtime());

if (-e $PackageConf) {
    my $keyword;
    my $value;
    open(FILE, "<$PackageConf");
    while (<FILE>) {
        chomp;
        ($keyword,$value)=split(/=/,$_);

        if ($keyword eq "hostname") {
            $my_hostname = $value;
        } elsif ($keyword eq "server_ip") {
            $server_ip = $value;
        } elsif ($keyword eq "server_port") {
            $server_port = $value;
        } elsif ($keyword eq "local_history") {
            $local_history = $value;
        } elsif ($keyword eq "history_separator") {
            $history_separator = $value;
        } elsif ($keyword eq "cpu_yellow") {
            $cpu_yellow = $value;
        } elsif ($keyword eq "cpu_red") {
            $cpu_red = $value;
        } elsif ($keyword eq "disk_yellow") {
            $disk_yellow = $value;
        } elsif ($keyword eq "disk_red") {
            $disk_red = $value;
        } elsif ($keyword eq "percent_physical_yellow") {
            $percent_physical_yellow = $value;
        } elsif ($keyword eq "percent_physical_red") {
            $percent_physical_red = $value;
        } elsif ($keyword eq "percent_actual_yellow") {
            $percent_actual_yellow = $value;
        } elsif ($keyword eq "percent_actual_red") {
            $percent_actual_red = $value;
        } elsif ($keyword eq "percent_swap_yellow") {
            $percent_swap_yellow = $value;
        } elsif ($keyword eq "percent_swap_red") {
            $percent_swap_red = $value;
        } elsif ($keyword eq "uptime_yellow") {
            $uptime_yellow = $value;
        }
    }
    close(FILE);
}

my $result;
my $test = '';

my ($model, $serial, $device_hostname, $firmware, $linux_version, $cpu_model, $system_temperature, $mac_addresses, $ip_addresses) = get_info();
my $info = "\n$linux_version\nModel: $model ($device_hostname)\nProcessor:$cpu_model\nSystem temperature: $system_temperature&deg;C\nSerial number: $serial\nFirmware: $firmware\nMAC address(es): $mac_addresses\nIP address(es): $ip_addresses\n\n$PackageNiceName $PackageVersion, $PackageLink\n\n";

my $prefix = $datetime." [".$my_hostname."] - ";

my ($uptime, $uptime_color) = get_uptime($uptime_yellow);
my ($cpu, $cpu_color, $cpu_load) = get_cpu($cpu_yellow,$cpu_red);
my ($disk, $disk_color, $disk_used) = get_disk($disk_yellow,$disk_red);
my ($memory, $memory_color, $memory_physical, $memory_actual, $memory_swap) = get_memory($percent_physical_yellow,$percent_physical_red,$percent_actual_yellow,$percent_actual_red,$percent_swap_yellow,$percent_swap_red);
my ($netstat, $netstat_color) = get_netstat();
my ($procs, $procs_color, $procs_count) = get_procs();
my ($who, $who_color, $who_count) = get_who();
my ($raid_state, $raid_state_color, $hdd_temperature) = get_raid_state();
my ($ups, $ups_color, $ups_state, $ups_load, $ups_runtime) = get_ups();

# Local history file
if ($local_history == 1) {
    my $line = "";

    unless (-e $PackageHistoryFile) {
        $line = "date_time";
        $line.= $history_separator;
        $line.= "uptime";
        $line.= $history_separator;
        $line.= "cpu_load";
        $line.= $history_separator;
        $line.= "physical_memory";
        $line.= $history_separator;
        $line.= "actual_memory";
        $line.= $history_separator;
        $line.= "swap_memory";
        $line.= $history_separator;
        $line.= "disk_usage";
        $line.= $history_separator;
        $line.= "processes";
        $line.= $history_separator;
        $line.= "ups_state";
        $line.= $history_separator;
        $line.= "ups_load";
        $line.= $history_separator;
        $line.= "ups_runtime";
        open (DEBUGFILE, '>>'.$PackageHistoryFile);
        print DEBUGFILE "$line\n";
        close (DEBUGFILE);
        print "History header written in $PackageHistoryFile\n";
    }

    $line = $datetime;
    $line.= $history_separator;
    $line.= $uptime;
    $line.= $history_separator;
    $line.= $cpu_load;
    $line.= $history_separator;
    $line.= $memory_physical;
    $line.= $history_separator;
    $line.= $memory_actual;
    $line.= $history_separator;
    $line.= $memory_swap;
    $line.= $history_separator;
    $line.= $disk_used;
    $line.= $history_separator;
    $line.= $procs_count;
    $line.= $history_separator;
    $line.= $ups_state;
    $line.= $history_separator;
    $line.= $ups_load;
    $line.= $history_separator;
    $line.= $ups_runtime;

    open (DEBUGFILE, '>>'.$PackageHistoryFile);
    print DEBUGFILE "$line\n";
    close (DEBUGFILE);
    print "History data written in $PackageHistoryFile\n";

    unless (-e $PackageHistoryFileLink)
    {
        $result = `ln -sf $PackageHistoryFile $PackageHistoryFileLink`;
    }
}


if (($my_hostname ne "") && ($server_ip ne "")) {
    $test = "cpu";
    send_info($server_ip, $server_port, $my_hostname,$test,$cpu_color,$prefix."up: ".convert_time_to_days($uptime).", $who_count users, $procs_count procs, load=$cpu_load\n".$cpu.$info."\n\n".$procs);

    $test = "disk";
    send_info($server_ip, $server_port, $my_hostname,$test,$disk_color,$prefix."$test\n".$disk.$info);

    $test = "memory";
    send_info($server_ip, $server_port, $my_hostname,$test,$memory_color,$prefix."$test\n".$memory.$info);

    $test = "netstat";
    send_info($server_ip, $server_port, $my_hostname,$test,$netstat_color,$prefix."$test\n".$info.$netstat);

    $test = "procs";
    send_info($server_ip, $server_port, $my_hostname,$test,$procs_color,$prefix."$test\n"."No process to check\n".$info.$procs_count." processes\n".$procs);

    $test = "raid";
    send_info($server_ip, $server_port, $my_hostname,$test,$raid_state_color,$prefix."$test\n".$raid_state.$info);

    $test = "temperature";
    my $temperature_information = "Status\tDevice\t\tTemp(C)\tTemp(F)\n";
    $temperature_information.= "---------------------------------------\n";
    $temperature_information.= "&green\tsystem\t\t".$system_temperature."\t".int (($system_temperature * 9 / 5) + 32)."\n";
    $temperature_information.= ($hdd_temperature);
    $temperature_information.= "---------------------------------------\n";

    send_info($server_ip, $server_port, $my_hostname,$test,"green",$prefix."$test\n".$temperature_information.$info);

    $test = "ups";
    if ($ups_color ne "clear") {
        send_info($server_ip, $server_port, $my_hostname,$test,$ups_color,$prefix."$test\n".$ups.$info);
    }

    $test = "uptime";
    send_info($server_ip, $server_port, $my_hostname,$test,$uptime_color,$prefix."$test\n".convert_time($uptime)."\n".$info);

    $test = "who";
    my $sword = "";
    if ($who_count > 1) {
      $sword = "s";
    }
    send_info($server_ip, $server_port, $my_hostname,$test,$who_color,$prefix."$test\n".$who."\n".$who_count." user".$sword."\n".$info);
} else {
    open (DEBUGFILE, '>>'.$PackageLog);
    print DEBUGFILE "$datetime\n";
    print DEBUGFILE "Configuration not fully completed to send data to a server.\n";
    close (DEBUGFILE);
}